"use strict";
/* ---------------------------------------------------------------- */
/* --------------------------- HELPERS ---------------------------- */
/* ---------------------------------------------------------------- */
Object.defineProperty(exports, "__esModule", { value: true });
exports.metadata = exports.useCache = void 0;
function useCache(cache, fn, getKey) {
    return (...args) => {
        const key = getKey(...args);
        const result = cache.get(key);
        if (!!result)
            return result;
        else {
            const newResult = fn(...args);
            cache.set(key, newResult);
            return newResult;
        }
    };
}
exports.useCache = useCache;
var metadata;
(function (metadata) {
    function createClass(parent, name) {
        return { [name]: class extends parent {
            } }[name];
    }
    metadata.createClass = createClass;
    function isParameterProperties(meta) {
        return meta && meta.kind === "Property" && meta.isParameter;
    }
    metadata.isParameterProperties = isParameterProperties;
    function isCallback(type) {
        return typeof type === "function" && !type.prototype;
    }
    metadata.isCallback = isCallback;
    function isConstructor(value) {
        return ("" + value).indexOf("class") == 0;
    }
    metadata.isConstructor = isConstructor;
    function isCustomClass(type) {
        switch (type) {
            case Boolean:
            case String:
            case Array:
            case Number:
            case Object:
            case Date:
                return false;
            default:
                return true;
        }
    }
    metadata.isCustomClass = isCustomClass;
    function getMethods(meta) {
        return meta.methods.map(x => ({
            name: x.name,
            type: x.returnType,
            pars: x.parameters
                .map(p => ({ name: p.name, type: p.type }))
        }));
    }
    metadata.getMethods = getMethods;
    function getProperties(meta) {
        return meta.properties.map(x => ({
            name: x.name,
            type: x.type
        }));
    }
    metadata.getProperties = getProperties;
})(metadata || (metadata = {}));
exports.metadata = metadata;
