"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reflect = void 0;
const decorators_1 = require("./decorators");
const helpers_1 = require("./helpers");
const parser_1 = require("./parser");
const walker_1 = require("./walker");
function reflectClass(target) {
    const visitorOrder = [
        walker_1.visitors.addsDesignTypes,
        walker_1.visitors.addsDecorators,
        walker_1.visitors.addsTypeOverridden,
        walker_1.visitors.addsParameterProperties,
        walker_1.visitors.addsGenericOverridden,
        walker_1.visitors.addsTypeClassification,
        walker_1.visitors.removeIgnored,
    ];
    const visitor = (value, ctx) => visitorOrder.reduce((a, b) => !!a ? b(a, ctx) : a, value);
    return walker_1.walkClass(target, { visitor, classPath: [] });
}
function traverseObject(fn, name) {
    if (Array.isArray(fn))
        return;
    if (typeof fn === "object")
        return reflectObject(fn, name);
    if (typeof fn === "function" && helpers_1.metadata.isConstructor(fn))
        return reflectClass(fn);
    if (typeof fn === "function")
        return parser_1.parseFunction(fn);
}
function reflectObject(object, name = "module") {
    return {
        kind: "Object", name,
        members: Object.keys(object).map(x => traverseObject(object[x], x)).filter((x) => !!x)
    };
}
// --------------------------------------------------------------------- //
// ------------------------------- CACHE ------------------------------- //
// --------------------------------------------------------------------- //
const cacheStore = new Map();
const reflectObjectCached = helpers_1.useCache(cacheStore, reflectObject, x => x);
const reflectClassRecursiveCached = helpers_1.useCache(cacheStore, reflectClass, x => x);
function reflect(option) {
    if (typeof option === "string") {
        return reflectObjectCached(require(option));
    }
    else {
        return reflectClassRecursiveCached(option);
    }
}
exports.reflect = reflect;
// --------------------------------------------------------------------- //
// ----------------------------- DECORATORS ---------------------------- //
// --------------------------------------------------------------------- //
/**
 * Decorator that do nothing, intended to be able to inspect data type
 */
reflect.noop = decorators_1.noop;
/**
 * Ignore member from metadata generated
 */
reflect.ignore = decorators_1.ignore;
/**
 * Override type definition information. Useful to add type definition for some data type that is erased
 * after transfile such as Partial<Type> or ReadOnly<Type>
 *
 * If applied to parameter it will override the parameter type
 *
 * If applied to property it will override the property type
 *
 * if applied to method it will overrid the method return value
 * @param type The type overridden
 * @param info Additional information about type (readonly, partial etc)
 */
reflect.type = decorators_1.type;
/**
 * Mark all constructor parameters as properties
 */
reflect.parameterProperties = decorators_1.parameterProperties;
